# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 Luis Felipe Lopez Acevedo <felipe.lopac@gmail.com>.
# Copyright (c) 2012 MoinMoin team.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
    MoinMoin - Galaxia theme
    
    @copyright: 2012 Luis Felipe Lopez Acevedo, aka sirgazil
    @license: GNU GPL, see COPYING for details.
"""

from MoinMoin.config import url_prefix_static
from MoinMoin.theme import ThemeBase

class Theme(ThemeBase):

    name = "galaxia"

    def header(self, d, **kw):
        """ Assemble wiki header

        @param d: parameter dictionary
        @rtype: unicode
        @return: page header html
        """
        html = [
            # Pre header custom html
            self.emit_custom_html(self.cfg.page_header1),

            # Header
            u'<div id="header">',
            self.logo(),
            self.human_menu(d),
            self.searchform(d),
            self.username(d),
            u'<div id="locationline">',
            self.interwiki(d),
            #self.title(d),
            u'</div>',
            self.breadcrumbs(d),
            #self.navibar(d),
            #self.trail(d),
            #u'<hr id="pageline">',
            u'<div id="pageline"><hr style="display:none;"></div>',
            self.msg(d),
            self.editbar(d),
            self.language_widget(d),
            u'</div>',

            # Post header custom html (not recommended)
            self.emit_custom_html(self.cfg.page_header2),

            # Start of page
            self.startPage(),
        ]
        return u'\n'.join(html)
    
    def human_menu(self, d, **kw):
        """ Assemble easy to read main menu for the header
        
        @param d: parameter dictionary
        @rtype: unicode
        @return: page main menu html
        """
        html = [
            u'<ul id="human-menu">',
            u'<li><a href="/Downloads">Downloads</a></li>',
            u'<li><a href="/Documentation">Documentation</a></li>',
            u'<li><a href="/Support">Support</a></li>',
            u'<li><a href="/Development">Development</a></li>',
            u'</ul>',
        ]
        return u'\n'.join(html)
    
    def breadcrumbs(self, d, **kw):
        """ Assemble a page path according to the page name
        
        @param d: parameter dictionary
        @rtype: unicode
        @return: page path html
        """
        html = [
            u'<div id="breadcrumbs">',
            u'<a id="path-root" href="/">',
            u'<img src="%s/galaxia/img/path-root.png" alt="Root">' % url_prefix_static,
            u'</a>',
        ]
        
        pages = d['page'].page_name.split('/')
        for page in pages:
            link = d['page'].page_name.partition(page)[0] + page
            html += [
                u'<a class="path-child" href="/' + link + '">' + page + '</a>',
            ]

        html += [
            u'</div>',
        ]
        
        return u'\n'.join(html)
    
    def editorheader(self, d, **kw):
        """ Assemble wiki header for editor

        @param d: parameter dictionary
        @rtype: unicode
        @return: page header html
        """
        html = [
            # Pre header custom html
            self.emit_custom_html(self.cfg.page_header1),

            # Header
            u'<div id="header">',
            self.logo(),
            self.human_menu(d),
            self.searchform(d),
            self.username(d),
            self.msg(d),
            u'</div>',

            # Post header custom html (not recommended)
            self.emit_custom_html(self.cfg.page_header2),

            # Start of page
            self.startPage(),
        ]
        return u'\n'.join(html)


    def language_widget(self, d, **kw):
        """ Assemble a custom language widget.
        
        This widget allows users to select translations available for the
        current page.
        
        @param d: parameter dictionary
        @rtype: unicode
        @return: language widget in html
        """        
        html = [
            u'<div id="lang-widget">',
            u'<div id="lang-list">',
            u'<div>Translations</div>',
        ]
        
        langs = self.cfg.translation_languages        
        for code in sorted(langs):
            html += [
                self.translation_link(d, code, langs[code]),
            ]
        
        html += [
            u'</div>',
            u'</div>',
        ]
        
        return u'\n'.join(html)
    
    
    def translation_link(self, d, lang_code, lang_name):
        """ Assamble a link to a translation in the given language.
        
        The link URL is formed from the current page URL.
        
        @param d: parameter dictionary
        @param lang_code: two-letter code of the language
        @param lang_name: complete name of the language
        @rtype: unicode
        @return: translation html link
        """
        link_text = u'%s (%s)' % (lang_name, lang_code)
        link_url = "#"
        
        page = d['page'].page_name.split('/')[-1]
        extension = page.split('.')[-1]
        
        # Remove any language code from the current page URL
        if len(extension) == 2 and extension.isalpha() and extension.islower():
            page_lang_code = '.%s' % page.split('.')[-1]
            page_url = d['page'].page_name.replace(page_lang_code, '')
        else:
            page_url = d['page'].page_name
        
        # Format translation link
        if lang_code == self.cfg.language_default:
            link_url = page_url
        else:
            link_url = u'%s.%s' % (page_url, lang_code)
        
        return u'<a href="/%s">%s</a>' % (link_url, link_text)
    

    def gns_footer(self, d, **kw):
        """ Assemble a custom gNewSense footer.
        
        @param d: parameter dictionary
        @rtype: unicode
        @return: gNewSense footer in html
        """        
        html = [
            u'<div id="gns-footer">',
            u'<p>',
            u'gNewSense is a project developed by volunteers all over the ',
            u'world and it\'s supported by the ',
            u'<a href="http://www.fsf.org/">Free Software Foundation</a>. ',
            u'The content in this Web site can be used as follows: ',
            u'All documentation is available under the terms of the ',
            u'<a href="http://www.gnu.org/copyleft/fdl.html">',
            u'GFDL with no invariant sections',
            u'</a>. ',
            u'<a href="/LicenceInformationUpdate/LicenceInformationUpdate">'
            u'(note on the license)',
            u'</a>. ',
            u'Artwork is Free Cultural Work and is available under the terms ',
            u'of the '
            u'<a href="http://creativecommons.org/licenses/by-sa/3.0/">',
            u'cc-by-sa</a> license.',
            u'</p>',
            u'</div>',
        ]
        
        return u'\n'.join(html)


    def footer(self, d, **keywords):
        """ Assemble wiki footer

        @param d: parameter dictionary
        @keyword ...:...
        @rtype: unicode
        @return: page footer html
        """
        page = d['page']
        html = [
            # End of page
            self.pageinfo(page),
            self.endPage(),

            # Pre footer custom html (not recommended!)
            self.emit_custom_html(self.cfg.page_footer1),

            # Footer
            u'<div id="footer">',
            self.editbar(d),
            self.gns_footer(d),
            self.credits(d),
            self.showversion(d, **keywords),
            u'</div>',

            # Post footer custom html
            self.emit_custom_html(self.cfg.page_footer2),
            ]
        return u'\n'.join(html)


def execute(request):
    """
    Generate and return a theme object

    @param request: the request object
    @rtype: MoinTheme
    @return: Theme object
    """
    return Theme(request)

